#define cbi(REG8,BITNUM) REG8 &= ~(_BV(BITNUM))
#define sbi(REG8,BITNUM) REG8 |= _BV(BITNUM)


typedef unsigned char byte;
typedef unsigned int word;

void InitPorts(void);
void TxRS485Packet(byte ID, byte Instruction, byte ParameterLength, byte *Parameter) ;
byte *RxRS485Packet(byte ID) ;
byte *RxRS485PacketIntr(byte ID);
unsigned long Timer(byte mode);


#define TIMRES_16000 1 // Timer Resolution: 16000 ticks per ms
#define TIMRES_2000 2  // Timer Resolution: 2000 ticks per ms
#define TIMRES_250 3   // Timer Resolution: 250 ticks per ms
void Robotis_SetTorqueLimit(short limit, byte eeprom_too, byte servo);
void Robotis_SetAngleLimit(short min, short max, byte servo);
void Robotis_SyncWrite(byte start_address, byte nbytes, byte nservos, byte *bytes, byte *servos);
void Robotis_RegWrite(byte start_address, byte nbytes, byte *bytes);
byte* Robotis_ReadMemory(byte servo, byte start_address, byte nbytes);
byte Robotis_Ping(byte servo);

//--- Control Table Address ---
//EEPROM AREA
#define P_MODEL_NUMBER_L      0
#define P_MODOEL_NUMBER_H     1
#define P_VERSION             2
#define P_ID                  3
#define P_BAUD_RATE           4
#define P_RETURN_DELAY_TIME   5
#define P_CW_ANGLE_LIMIT_L    6
#define P_CW_ANGLE_LIMIT_H    7
#define P_CCW_ANGLE_LIMIT_L   8
#define P_CCW_ANGLE_LIMIT_H   9
#define P_SYSTEM_DATA2        10
#define P_LIMIT_TEMPERATURE   11
#define P_DOWN_LIMIT_VOLTAGE  12
#define P_UP_LIMIT_VOLTAGE    13
#define P_MAX_TORQUE_L        14
#define P_MAX_TORQUE_H        15
#define P_RETURN_LEVEL        16
#define P_ALARM_LED           17
#define P_ALARM_SHUTDOWN      18
#define P_OPERATING_MODE      19
#define P_DOWN_CALIBRATION_L  20
#define P_DOWN_CALIBRATION_H  21
#define P_UP_CALIBRATION_L    22
#define P_UP_CALIBRATION_H    23

#define P_TORQUE_ENABLE         (24)
#define P_LED                   (25)
#define P_CW_COMPLIANCE_MARGIN  (26)
#define P_CCW_COMPLIANCE_MARGIN (27)
#define P_CW_COMPLIANCE_SLOPE   (28)
#define P_CCW_COMPLIANCE_SLOPE  (29)
#define P_GOAL_POSITION_L       (30)
#define P_GOAL_POSITION_H       (31)
#define P_GOAL_SPEED_L          (32)
#define P_GOAL_SPEED_H          (33)
#define P_TORQUE_LIMIT_L        (34)
#define P_TORQUE_LIMIT_H        (35)
#define P_PRESENT_POSITION_L    (36)
#define P_PRESENT_POSITION_H    (37)
#define P_PRESENT_SPEED_L       (38)
#define P_PRESENT_SPEED_H       (39)
#define P_PRESENT_LOAD_L        (40)
#define P_PRESENT_LOAD_H        (41)
#define P_PRESENT_VOLTAGE       (42)
#define P_PRESENT_TEMPERATURE   (43)
#define P_REGISTERED_INSTRUCTION (44)
#define P_PAUSE_TIME            (45)
#define P_MOVING (46)
#define P_LOCK                  (47)
#define P_PUNCH_L               (48)
#define P_PUNCH_H               (49)

//--- Instruction ---
#define INST_PING           0x01
#define INST_READ           0x02
#define INST_WRITE          0x03
#define INST_REG_WRITE      0x04
#define INST_ACTION         0x05
#define INST_RESET          0x06
#define INST_DIGITAL_RESET  0x07
#define INST_SYSTEM_READ    0x0C
#define INST_SYSTEM_WRITE   0x0D
#define INST_SYNC_WRITE     0x83
#define INST_SYNC_REG_WRITE 0x84

//#define CLEAR_BUFFER gbRxBufferReadPointer = gbRxBufferWritePointer
#define DEFAULT_RETURN_PACKET_SIZE 6
#define BROADCASTING_ID 0xfe // used in conjuction with PING and SYNC_WRITE

// ATmega128 table 82 page 198 [Asynchronous Double Speed Mode (U2X = 1)]
#define BAUD_57600 34   //BAUD = 57600bps at U2X=1, Fosc=16MHz => UBRR=34  
#define BAUD_115200 16   //BAUD = 115200bps at U2X=1, Fosc=16MHz => UBRR=16  
#define BAUD_1000000 1   //BAUD = 1000000bps at U2X=1, Fosc=16MHz => UBRR=1  

/* UART TX in Atmega has a double buffer mechanism.
 * The "buffer" (UDRn) can hold a byte to be sent while a previous byte is being sent
 * UCSRnA[6] is set when the "buffer" can receive a new value while previous one is being shifted out
 * UCSRnA[5] is set when the value has been shifted out and buffer is empty
 */

#if defined(_CM5_)
#define SET_RS485_TXD PORTE &= ~_BV(PE3),PORTE |= _BV(PE2)  //_485_DIRECTION = 1
#define SET_RS485_RXD PORTE &= ~_BV(PE2),PORTE |= _BV(PE3)  //PORT_485_DIRECTION = 0
#define BIT_RS485_RX_DDR  0x08  //Port E - responsible for enabling RS485 RX 
#define BIT_RS485_TX_DDR  0x04  //Port E - responsible for enabling RS485 TX
#define CHECK_TXRS485_READY		bit_is_set(UCSR0A,5) // "The UDREn flag indicates if the transmit buffer (UDRn) is ready to receive new data. If UDREn is one, the buffer is empty, and therefore ready to be written"
#define CHECK_TXRS485_FINISH	bit_is_set(UCSR0A,6) // "TXCn: USART Transmit Complete flag bit is set when the entire frame in the Transmit Shift Register has been shifted out and there are no new data currently present in the transmit buffer (UDRn)."
#define CHECK_RXRS485_READY		bit_is_set(UCSR0A,7) // "This flag bit is set when there are unread data in the receive buffer and cleared when the receive buffer is empty"
#define TXRS485_DATA			(UDR0)
#define RXRS485_DATA			(UDR0)
#define SET_TXRS485_FINISH   	sbi(UCSR0A,6) // "The TXCn flag bit is automatically cleared when a transmit complete interrupt is executed, or it can be cleared by writing a one to its bit location."
#define RESET_TXRS485_FINISH 	cbi(UCSR0A,6) // ?


#define CHECK_TXPC_READY		bit_is_set(UCSR1A,5) 
#define TXPC_DATA				(UDR1)
#define CHECK_RXPC_READY		bit_is_set(UCSR1A,7)
#define RXPC_DATA				(UDR1)
#define SET_TXPC_FINISH			sbi(UCSR1A,6)
#define RESET_TXPC_FINISH		cbi(UCSR1A,6)
#define CHECK_TXPC_FINISH bit_is_set(UCSR1A,6)
#define BIT_ZIGBEE_RESET               PD4  //out : default 1 //PORTD
#define BIT_ENABLE_RXD_LINK_PC         PD5  //out : default 1
#define BIT_ENABLE_RXD_LINK_ZIGBEE     PD6  //out : default 0
#define BIT_LINK_PLUGIN                PD7  //in, no pull up
void TxPC(byte bTxdData);
void TxPC32Dec(long lLong);
void TxPCString(byte *str);
void TxPCPrintBaudrate(void);
void TxPCHex(byte b);
byte RxPC(void);
void InitSerialPC(byte bBaudrate, byte bInterrupt);
void InitSerialRS485(byte bBaudrate, byte bInterrupt);

#elif defined(_Pepper_)
#define SET_RS485_TXD PORTD &= ~_BV(PD6),PORTD |= _BV(PD7)  // 485 transmit mode
#define SET_RS485_RXD PORTD &= ~_BV(PD7),PORTD |= _BV(PD6)  // 485 receive mode
#define BIT_RS485_RX_DDR  _BV(PD6)  //Port D - responsible for enabling RS485 RX 
#define BIT_RS485_TX_DDR  _BV(PD7)  //Port D - responsible for enabling RS485 TX
#define CHECK_TXRS485_READY		bit_is_set(UCSRA,5)
#define CHECK_RXRS485_READY		bit_is_set(UCSRA,7)
#define CHECK_TXRS485_FINISH	bit_is_set(UCSRA,6)
#define TXRS485_DATA		(UDR)
#define RXRS485_DATA		(UDR)
#define SET_TXRS485_FINISH   sbi(UCSRA,6) // reads 0 after this operation, may create interrupt
#define RESET_TXRS485_FINISH cbi(UCSRA,6)
void InitSerialRS485(byte bInterrupt);

#elif defined(_CM2_)
#define RS485_TXD PORTE |= _BV(PE2); //_485_DIRECTION = 1
#define RS485_RXD PORTE &= ~_BV(PE2);//PORT_485_DIRECTION = 0
#endif

//#define TXD0_FINISH  UCSR0A,6  //This bit is for checking TxD Buffer in CPU is empty or not.
//#define TXD1_FINISH	 UCSR1A,6 








