/*
 * bioloid_cm5.c
 * Robotis CM5-side Robotis protocol implementation
 */
#if defined(_CM5_)


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "robotis.h"


/*
 * portInitialize - sets the data directions of serial ports
 */
void InitPorts(void)
{

	DDRA = DDRB = DDRC = DDRD = DDRE = DDRF = 0;  //Set all port to input direction first.
	PORTB = PORTC = PORTD = PORTE = PORTF = PORTG = 0x00; //PortData initialize to 0
	cbi(SFIOR,2); //All Port Pull Up ready
	DDRE |= (BIT_RS485_RX_DDR|BIT_RS485_TX_DDR); //set output data direction for RS485 enablers

	DDRD |= (BIT_ZIGBEE_RESET|BIT_ENABLE_RXD_LINK_PC|BIT_ENABLE_RXD_LINK_ZIGBEE);
	PORTD &= ~_BV(BIT_LINK_PLUGIN); // no pull up
	PORTD |= _BV(BIT_ZIGBEE_RESET)|_BV(BIT_ENABLE_RXD_LINK_PC)|_BV(BIT_ENABLE_RXD_LINK_ZIGBEE);

	SET_RS485_RXD; // set the RS485 interface to Rx by default
}




/*
 * InitSerialRS485 - set the i/o pins to talk to servos
 * Interrupt - if 1 then input will be interrupt driven
 *             gInterruptBuffer[gInterruptBufferPointer++] will continously take new read values
 * Baudrate - used in conjunction with prescaler...
 */
void InitSerialRS485(byte Baudrate, byte Interrupt)
{

    UBRR0H = 0;
	UBRR0L = Baudrate; 
    UCSR0A = 0x02; // U2X: Double the USART transmission speed
	UCSR0B = 0x18; // Receiver and Transmitter enable
    if(Interrupt) sbi(UCSR0B,7); // RxD interrupt enable
    UCSR0C = 0x06; // 8 bit data, no parity
	UDR0 = 0xFF; // puts a value in the buffer (dont know why?)
    SET_TXRS485_FINISH; // Note. set 1, then 0 is read
	if(Interrupt) sei();
}


/*
 * InitSerialPC - set the i/o pins to talk to PC
 * Interrupt - if 1 then input will be interrupt driven
 *             gInterruptBuffer[gInterruptBufferPointer++] will continously take new read values
 * Baudrate - used in conjunction with prescaler...
 */
void InitSerialPC(byte Baudrate, byte Interrupt)
{
    UBRR1H = 0;
	UBRR1L = Baudrate; 
    UCSR1A = 0x02;
	UCSR1B = 0x18;
    if(Interrupt) sbi(UCSR1B,7);
    UCSR1C = 0x06;
	UDR1 = 0xFF;
    SET_TXPC_FINISH;
}


/*
TXPC32Dec() - print 32bit decimal value to PC port
*/
void TxPC32Dec(long lLong)
{
  byte bCount, bPrinted;
  long lTmp,lDigit;
  bPrinted = 0;
  if(lLong < 0)
  {
    lLong = -lLong;
    TxPC('-');
  }
  lDigit = 1000000000L;
  for(bCount = 0; bCount < 9; bCount++)
  {
    lTmp = (byte)(lLong/lDigit);
    if(lTmp) 
    {
      TxPC(((byte)lTmp)+'0');
      bPrinted = 1;
    }
    else if(bPrinted) TxPC(((byte)lTmp)+'0');
    lLong -= ((long)lTmp)*lDigit;
    lDigit = lDigit/10;
  }
  lTmp = (byte)(lLong/lDigit);
  /*if(lTmp)*/ TxPC(((byte)lTmp)+'0');
}

/*
TxD8Hex() - print byte in hex into PC serial port
ex" 0x1a -> '1' 'a'.
*/
void TxPCHex(byte b)
{
  byte Tmp;

  Tmp =((byte)(b>>4)&0x0f) + (byte)'0';
  if(Tmp > '9') Tmp += 7;
  TxPC(Tmp);
  Tmp =(byte)(b & 0x0f) + (byte)'0';
  if(Tmp > '9') Tmp += 7;
  TxPC(Tmp);
}


/* 
 * TxPCString() prints ACSII data to PC serial link.
 */
void TxPCString(byte *str)
{
  while(*str)
  {
    TxPC(*str++);
  }
}


/*
 * RxPC() read data from UART1 (PC serial link).
 * RxPC() return Read data.
 */
byte RxPC(void)
{
	while(!CHECK_RXPC_READY);
	return(RXPC_DATA);
}


/*
 * TxPC() write data to UART1 (PC serial link).
 */
void TxPC(byte bTxdData)
{
	while(!CHECK_TXPC_READY);
	TXPC_DATA = bTxdData;
}




/* Print value of Baud Rate.
*/
void TxPCPrintBaudrate(void)
{
	TxPCString("\r\nRS232:");
	TxPC32Dec((16000000L/8L)/((long)UBRR1L+1L) );
	TxPCString(" BPS,");
	TxPCString(" RS485:");
	TxPC32Dec((16000000L/8L)/((long)UBRR0L+1L) );
	TxPCString(" BPS\r\n");
}

#endif _CM5_

